100 '       Lawson Labs
110 '       15 Phoenixville Pike
120 '       Malvern, PA  19355
130 '       800 321-5355  610 725-8800
140 '
150 '       Sample driver program for Model 141 20-bit A/D converter board
160 '       Usable with interpreted BASIC, QuickBASIC or QBASIC
170 '                   Rev. 0           February 3, 1992
180 '
190 '       Card base address :
200 '             The delta sigma card requires 4 consecutive port addresses.
210 '             To obtain the base address from the DIP switch settings, use
220 '             0 for ON and 1 for OFF in the following formula :
230 '
240 '  8 * (SW7 * 64 + SW6 * 32 + SW5 * 16 + SW4 * 8 + SW3 * 4 + SW2 * 2 + SW1)
250 '
260 '             The default setting is Hex 230 (decimal 560)
270 '               SW7, SW3 and SW2 are OFF and all others are ON
280 '
290 '       Conversion Clock :
300 '             The jumper in the upper left hand corner of the board sets
310 '             the clock rate for the card.
320 '             The jumper in position 1 yields 224 KHz.
330 '             The jumper in position 2 yields 895 KHz.
340 '             The jumper in position 3 yields 3.58 MHz.
350 '             The corner frequency at 224 KHz is 0.55 Hz, with
360 '             calibrations requiring 10 seconds to complete.
370 '             The update rate is 218 Hz.
380 '             The corner frequency at 895 KHz is 2.2 Hz, with calibrations
390 '             requiring 2.5 seconds to complete. Update rate= 874 Hz.
400 '             The corner frequency at 3.58 MHz is 8.7 Hz, with calibrations
410 '             requiring 0.7 seconds to complete. Update rate= 3.5 KHz.
420 '
430 '       Inputs:
440 '             maximum differential input        +/- 5.0 volts
450 '             common mode voltage range         +/- 6.0 volts
460 '             overvoltage protection             120 volts continuous
470 '
480 '     20-bit resolution with no missing codes. +/- 4 counts reproducibility
490 '                                         (bipolar mode with 895 KHz clock)
500 '
510 '  READS:     Base address = low byte of answer
520 '             Base + 1     = middle byte of answer
530 '             Base + 2     = high byte of answer
540 '             Base + 3     = status  most sig bit= CONVERTING
550 '                                4th most sig bit= DIGITAL INPUT
560 ' WRITES:     Base address = control word
570 '                 least sig nibble is  A, B, C & D output
580 '                 bit 7 is BIPOLAR RANGE
590 '                 bit 6 is CALIBRATE
600 '                 bits 5 & 4 are Channel Selection code
610 '                      code 00 selects input channel 0
620 '                      code 01 selects input channel 1
630 '                      code 10 selects the full scale calibration
640 '                      code 11 selects the offset (zero) calibration
650 '                 Base + 1       least sig bit is FREEZE
660 '
670 KEY OFF: CLS                ' start with screen clear
680 ADC% = &H230 : GOSUB 1680   ' get board base address
690 MIN = 5 : MAX = -5          ' initialize max and min
700 BP% = 128                   ' BIPOLAR MODE (use 0 for unipolar)
710 CH% = 0                     ' default input channel
720 CC% = 0                     ' default output control code
730 AVG% = 1                    ' default number of counts to average
740 GOSUB 1230                  ' print column headings, etc.
750 GOSUB 900                   ' offset, fullscale calibration
760 H = 0: M = 0: L = 0         ' initialize data readout
770 FOR N = 1 TO AVG%           ' average loop counter
780 OUT ADC% + 1, 1             ' assert freeze
790 H = H + (INP(ADC% + 2) AND &HF) ' get the 4 most significant bits
800 M = M + INP(ADC% + 1)       ' get middle byte
810 L = L + INP(ADC%)           ' get low byte
820 OUT ADC% +1, 0              ' unfreeze
830 NEXT                        ' get next data for averaging
840 GOSUB 1080                  ' convert data to volts and print
850 A$ = INKEY$                 ' check for keypress
860 IF A$ = " " THEN MIN = 5 : MAX = -5 ' "SPACE" hit, reset max and min
870 IF A$ = "m" OR A$ = "M" THEN GOSUB 1290 ' offer menu
880 IF CSRLIN = 23 THEN LOCATE 3, 1 ' wrap around to top of screen
890 GOTO 760                    ' do another average loop
900 PRINT "Calibrating"; : LOCATE , 1 ' beginning of calibration routines
910 OUT ADC% + 1, 1             ' freeze
920 OUT ADC%, 48 + BP%          ' offset calibration code
930 OUT ADC%, 48 + 64 + BP%     ' set calibration bit
940 START=TIMER                 ' remember the time
950 OUT ADC%, 48 + BP%          ' clear calibration bit
960 OUT ADC% + 1, 0             ' unset freeze
970 IF INP(ADC%+3) > 127 THEN GOTO 970 'wait til done
980 OUT ADC% + 1, 1             ' freeze
990 OUT ADC%, 32 + BP%          ' fullscale calibration code
1000 OUT ADC%, 32 + 64 + BP%    ' set calibration bit
1010 OUT ADC%, 32 + BP%         ' clear calibration bit
1020 OUT ADC% + 1, 0            ' unset freeze
1030 IF INP(ADC% + 3) > 127 THEN GOTO 1030 'wait til done
1040 CALTIM = TIMER - START     ' compute time required for calibration
1050 OUT ADC%, CH% * 16 + BP% + CC% ' select channel
1060                            ' wait for settling can go here
1070 RETURN                     ' end of calibration
1080 REM volts                  ' start of count to volts conversion routine
1090 COUNT! = L / AVG% + (M / AVG%) * 256 + (H / AVG%) * 65536! ' merge counts
1100 PRINT USING "########"; COUNT!;            ' print count
1110 VOLTS = COUNT! * 9.536743E-06 - 5          ' convert to volts
1120 IF BP% =0 THEN VOLTS = COUNT! * 4.768372E-06 ' for unipolar
1130 PRINT USING "#####.#####"; VOLTS;          ' print volts
1140 IF VOLTS < MIN THEN MIN = VOLTS            ' set minimum reading
1150 IF VOLTS > MAX THEN MAX = VOLTS            ' set maximum reading
1160 PRINT USING "#####.#####"; MIN;            ' print minimum reading
1170 PRINT USING "#####.#####"; MAX;            ' print maximum reading
1180 PRINT USING "#####.#####"; MAX - MIN       ' print the spread
1190 ROW%=CSRLIN                                ' remember what row we're on
1200 LOCATE 25,77: IF (INP (ADC%+3) AND 16) THEN PRINT "OFF"; ELSE PRINT "ON ";
1210 LOCATE ROW%, 1                             ' now go back there
1220 RETURN                                     ' end of conversion routine
1230 CLS : LOCATE 25, 1                         ' menu/status line at bottom
1240 PRINT "Press space to reset Max and Min or M for Menu";
1250 LOCATE 25, 63 : PRINT "Digital Input ";    ' for digital input ON/OFF
1260 LOCATE 1, 1: PRINT "  Count     Volts      Min         Max        Spread";
1270 LOCATE 3, 1                                ' first line for analog data
1280 RETURN
1290 CLS : LOCATE 25, 10                        ' menu screen option
1300 PRINT "A/D clock = ";                      ' show A/D clock rate sensed
1310 IF CALTIM >= 5 THEN PRINT "224 KHz";       ' jumper position 1
1320 IF (CALTIM > 1.5 AND CALTIM < 5) THEN PRINT "895 KHz"; ' jumper position 2
1330 IF CALTIM <= 1.5 THEN PRINT "3.58 MHz";    ' jumper position 3
1340 LOCATE 1, 1                                ' go back to top of screen
1350 PRINT "(1)  # points to average (1-32767):    ";AVG%  ' 1-32767 points
1360 PRINT "(2)  input channel                     ";CH%   ' channel 0 or 1
1370 PRINT "(3)  digital output code (0-15)        ";CC%   ' codes 0 - 15
1380 PRINT "(4)  input mode                        ";  ' bipolar or unipolar
1390 IF BP% = 128 THEN PRINT "bipolar"          ' -2.5 to +2.5 volt range
1400 IF BP% = 0 THEN PRINT "unipolar"           ' 0 to +5 volt range
1410 PRINT "(5)  recalibrate"                   ' for temperature drift
1420 PRINT "(6)  exit program"                  ' done with program
1430 PRINT "(7)  resume operation"              ' continue measurements
1440 PRINT : INPUT "Select";Q% : PRINT          ' select menu choice
1450 IF Q% < 1 OR Q% > 7 THEN GOTO 1290         ' limit to valid selections
1460 IF Q% = 7 THEN GOSUB 1230 : RETURN         ' redraw headings and exit menu
1470 ON Q% GOSUB 1490,1520,1560,1600,1640,1670  ' execute selected choice
1480 GOTO 1290                                  ' offer the menu again
1490 INPUT "# points to average "; AVG%         ' choose # points to average
1500 MIN = 5 : MAX = -5                         ' reset max and min
1510 RETURN
1520 IF CH% = 0 THEN CH% = 1 ELSE CH% = 0       ' change to other input channel
1530 OUT ADC%, CH% * 16 + BP% + CC%             ' select the new channel
1540 MIN = 5 : MAX = -5                         ' reset max and min
1550 RETURN
1560 INPUT "digital output code "; CC%          ' select new output code
1570 CC% = CC% MOD 16                           ' limit code to allowable range
1580 OUT ADC%, CH% * 16 + BP% + CC%             ' output the new code
1590 RETURN
1600 IF BP% = 128 THEN BP% = 0 ELSE BP% = 128   ' change to other mode
1610 CLS : GOSUB 900                            ' must recalibrate for new mode
1620 MIN = 5 : MAX = -5                         ' reset max and min
1630 RETURN
1640 CLS : GOSUB 900                            ' do the recalibration
1650 MIN = 5 : MAX = -5                         ' reset max and min
1660 RETURN
1670 CLS : LOCATE 23, 1 : END                   ' exit from the program
1680 PRINT "ADC address = "; ADC% ;"? (Y/N)"    ' start with default address
1690 A$ = INPUT$(1)                             ' want to change it?
1700 IF A$ = "Y" OR A$ = "y" THEN GOTO 1900     ' no, leave address alone
1710 IF A$ <> "N" AND A$ <> "n" THEN GOTO 1680  ' ignore invalid responses
1720 PRINT "Enter ADC% or DIP switch setting in binary."  ' prompt for new addr
1730 PRINT "(For binary, use 1 for ON and 0 for OFF. ";   '   in either binary
1740 PRINT " Start with switch #7)";                      '     or decimal
1750 INPUT A$                                   ' enter the new address
1760 IF LEN(A$) <> 7 THEN GOTO 1830             ' assume decimal if <> 7 chrs
1770 ADC% = 0                                   ' accumulate weighted binary
1780 FOR N% = 1 TO 7                            ' 7 bits to add up
1790 DIGIT% = (2 ^ (7 - N%)) * ABS(VAL(MID$(A$, N%, 1)) - 1)  ' weight the bit
1800 ADC% = ADC% + DIGIT%                       ' add it to the sum
1810 NEXT N%                                    ' do all of them
1820 ADC% = ADC% * 8: A$ = STR$(ADC%)           ' 8-byte boundaries for address
1830 IF LEN(A$) > 4 THEN GOTO 1720              ' if > 4, invalid
1840 ADC% = VAL(A$)                             ' assign value input to ADC%
1850 PRINT "ADC% in decimal="; ADC%,            ' display new value in decimal
1860 PRINT "ADC% in Hex="; HEX$(ADC%)           ' also display it in hex
1870 PRINT "Enter space to continue or Esc to re-enter."  ' allow another round
1880 A$ = INPUT$(1)                             ' select choice
1890 IF A$ = CHR$(27) THEN CLS : GOTO 1720      ' if Esc, go get a new value
1900 GOSUB 1980                                 ' test for presence of 141
1910 IF FOUND% THEN RETURN                      ' 141 found at ADC%, all done
1920 BEEP : PRINT "A Model 141 board was not";  ' beep and display message that
1930 PRINT " found at address "; ADC%; " (";    '   141 board not found at the
1940 PRINT HEX$(ADC%); " Hex)"                  '     current address
1950 PRINT "Enter space to exit program or Esc to re-enter."  ' allow exit here
1960 A$ = INPUT$(1)                             ' select choice
1970 IF A$ <> CHR$(27) THEN GOTO 1670 ELSE CLS : GOTO 1720  ' exit or try again
1980 PRINT "Testing for presence of A/D board . . ."  ' print message
1990 OUT ADC%, 128                              ' make sure not calibrating
2000 OUT ADC% + 1, 1                            ' assert the FREEZE bit
2010 IF INP(ADC% + 3) < 128 THEN FOUND% = 0 : RETURN  ' CONVERTING bit missing
2020 OUT ADC% + 1, 0                            ' now remove the FREEZE bit
2030 IF TIMER > 86390! THEN GOTO 2030           ' avoid imminent rollover
2040 START = TIMER                              ' remember the time
2050 IF TIMER < START+10 AND INP(ADC%+3) > 127 THEN GOTO 2050 ' wait and watch
2060 IF INP(ADC% + 3) > 127 THEN FOUND% = 0 : RETURN  ' spurious CONVERTING bit
2070 OUT ADC% + 1, 0                            ' start conversion if 140 board
2080 IF INP(ADC% + 3) > 127 THEN FOUND% = 0 : RETURN  ' must be a 140
2090 FOUND% = -1 : RETURN                       ' A/D card appears to be there
